'use strict';

const http = require('http');
const formidable = require('formidable');
const fs = require('fs');
const path = require('path');
const moment = require('moment');

module.exports = class HttpServer {

    constructor(config, watchers, homeDir) {
        this.update(config, watchers);
        this.maxFileSize = 25 * 1024 * 1024;
        this.tempFolder = homeDir + '/temp';
        if (fs.existsSync(this.tempFolder)) {
            this.deleteFolderRecursive(this.tempFolder);
        }
        fs.mkdirSync(this.tempFolder);
    }

    deleteFolderRecursive (path) {
        if (fs.existsSync(path)) {
            fs.readdirSync(path).forEach((file, index) => {
                var curPath = path + '/' + file;
                if (fs.lstatSync(curPath).isDirectory()) {
                    this.deleteFolderRecursive(curPath);
                } else {
                    fs.unlinkSync(curPath);
                }
            });
            fs.rmdirSync(path);
        }
    }

    update(config, watchers) {
        this.config = config;
        this.watchers = watchers;
    }

    notFound(res) {
        res.status = 404;
        res.end("Not found\r\n");
    }

    runServer(httpPort) {
        http.createServer((req, res) => {
            
            res.setHeader('Content-Type', 'text/plain');
            let current = this.getCurrent(req);
            if (current === null) {
                return this.notFound(res);
            }

            if (current.archive_file) {
                let debug = {};
                debug.body = "";
    
                req.on("data", (chunk) => {
                    debug.body += chunk.toString('utf-8');
                });
    
                req.on("end", () => {
                    debug.headers = req.headers;
                    debug.method = req.method;
                    debug.url = req.url;
                    console.log(debug);
                    fs.writeFile(path.join((process.env.HOME || process.env.USERPROFILE), '/rmconnect/debug/request_' + moment().format('YYYY-MM-DD-HHmmSSsss') + '.json'), JSON.stringify(debug, null, 2), (err) => {
                        if (err) console.error(err);
                    });
                });
    
            }
            
            this.loadForm(req).then((file) => {
                for (var i = 0, l = this.watchers.length; i < l; i++) {
                    if (this.watchers[i].key == current.currentKey) {
                        this.watchers[i].tryReadFile(file);
                        res.end("Processing "+file+"\r\n");
                        return;
                    }
                }
                this.notFound(res);
            }).catch((error) => {
                res.status = 500;
                res.end("Error: "+error+"\r\n");
            });
        }).listen(httpPort, () => {
            console.log('Server running on: http://127.0.0.1:'+httpPort);
        });
    }

    getCurrent (req) {
        if (req.method === 'POST') {
            for (let key in this.config) {
                if (this.config.hasOwnProperty(key)) {
                    if (this.config[key].active && this.config[key].useHttp && '/'+this.config[key].url === req.url) {
                        return this.config[key];
                    }
                }
            }
        }
        return null;
    }

    loadForm(req) {
        return new Promise((resolve, reject) => {
            var form = new formidable.IncomingForm();
            form.uploadDir = this.tempFolder;
            form.maxFieldsSize = this.maxFileSize;
            form.parse(req, (err, fields, files) => {
                if (Object.keys(files).length === 0) {
                    return reject('No file posted');    
                }
                if (err) {
                    return reject(err);
                }
                for (let file in files) {
                    if (files.hasOwnProperty(file)) {
                        let filePath = files[file].path.replace(/upload_/, '') + '/';
                        if (!fs.existsSync(filePath)) fs.mkdirSync(filePath);
                        filePath = filePath + files[file].name;
                        fs.renameSync(files[file].path, filePath);
                        resolve(filePath);
                        return;
                    }
                }
                reject();
            });
            form.on('error', (err) => {
                console.error(err);
                reject(err);
            })
        });
    }

};
